#!/usr/bin/python3

def gcd(a, b):
    while a and b:
        a, b = b, a % b
    return a or b

def sign(x):
    if not x:
        return 0
    return x // abs(x)

class Rational:
    def __init__(self, numerator=0, denominator=1):
        numerator = int(numerator)
        denominator = int(denominator)
        if not denominator:
            raise ZeroDivisionError

        s = sign(numerator * denominator)
        g = gcd(abs(numerator), abs(denominator))
        self.num = s * abs(numerator) // g
        self.den = abs(denominator) // g

    def __lt__(self, other):
        return self.num * other.den < other.num * self.den

    def __eq__(self, other):
        return not (self < other or other < self)

    def __le__(self, other):
        return self < other or self == other

    def __ne__(self, other):
        return not (self == other)

    def __gt__(self, other):
        return not (self <= other)

    def __ge__(self, other):
        return not (self < other)

    def __hash__(self):
        return self.num + 37 * self.den

    def __bool__(self):
        return bool(self.num)

    def __str__(self):
        if self.den == 1:
            return str(self.num)
        return "{}/{}".format(self.num, self.den)

    def __neg__(self):
        return Rational(-self.num, self.den)

    def __pos__(self):
        return self

    def __add__(self, other):
        if type(other) is not Rational:
            other = Rational(other)
        num = self.num * other.den + other.num * self.den
        den = self.den * other.den
        return Rational(num, den)

    def __radd__(self, other):
        return Rational(other) + self

    def __sub__(self, other):
        if type(other) is not Rational:
            other = Rational(other)
        return self + -other

    def __rsub__(self, other):
        return Rational(other) - self


def main():
    print(Rational(1, 2) + Rational(1, 2))
    print(Rational(1, 2) - Rational(1, 2))
    print(Rational(-1, 2) + 1)
    print(Rational(-1, 2) - 1)
    print(1 + Rational(-1, 2))
    print(1 - Rational(-1, 2))

if __name__ == "__main__":
    main()
