#!/usr/bin/python3
from collections import OrderedDict

class Bind:
    def __init__(self, f, **kwargs):
        self.callable = f
        self.binded_args = {}
        for k, v in kwargs.items():
            i = int(k.strip('_'))
            self.binded_args[i] = v

    def __call__(self, *args):
        return self.callable(*self._make_args(args))

    def _make_args(self, args):
        call_args = OrderedDict()
        next_pos, next_arg = 1, 0

        for pos, arg in sorted(self.binded_args.items()):
            while pos > next_pos:
                call_args[next_pos] = args[next_arg]
                next_pos += 1
                next_arg += 1
            call_args[next_pos] = self.binded_args[next_pos]
            next_pos += 1

        return [_ for _ in call_args.values()]

def main():
    def foo(a, b, c, d, e, f, g):
        print(a, b, c, d, e, f, g)

    Bind(foo, _1 = 'a', _3 = 'c', _5 = 'e', _7 = 'g')('b', 'd', 'f')

if __name__ == '__main__':
    main()
