#!/usr/bin/python3

def gcd(a, b):
    while a and b:
        a, b = b, a % b
    return a or b

def sign(x):
    if not x:
        return 0
    return x // abs(x)

class Rational:
    def __init__(self, numerator=0, denominator=1):
        numerator = int(numerator)
        denominator = int(denominator)
        if not denominator:
            raise ZeroDivisionError

        s = sign(numerator * denominator)
        g = gcd(abs(numerator), abs(denominator))
        self.num = s * abs(numerator) // g
        self.den = abs(denominator) // g

    def __lt__(self, other):
        return self.num * other.den < other.num * self.den

    def __eq__(self, other):
        return not (self < other or other < self)

    def __le__(self, other):
        return self < other or self == other

    def __ne__(self, other):
        return not (self == other)

    def __gt__(self, other):
        return not (self <= other)

    def __ge__(self, other):
        return not (self < other)

    def __hash__(self):
        return self.num + 37 * self.den

    def __bool__(self):
        return bool(self.num)

    def __str__(self):
        if self.den == 1:
            return str(self.num)
        return "{}/{}".format(self.num, self.den)


def main():
    from itertools import product

    def print_cmp(a, b):
        print("{} < {} = {}".format(a, b, a < b))
        print("{} <= {} = {}".format(a, b, a <= b))
        print("{} == {} = {}".format(a, b, a == b))
        print("{} >= {} = {}".format(a, b, a >= b))
        print("{} > {} = {}".format(a, b, a > b))

    a = Rational(1, 3)
    b = Rational(2, 6)
    c = Rational(4, 9)
    d = Rational(-4, 9)
    e = Rational(4, -9)

    for l, r in product([a, b, c, d, e], repeat=2):
        print_cmp(l, r)

if __name__ == "__main__":
    main()
